package com.aliyun.chatbot.channel.chatbot;

import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import lombok.extern.slf4j.Slf4j;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.sse.EventSource;
import okhttp3.sse.EventSources;

/**
 * @author kelin
 * @date 2024/6/11
 */
@Slf4j
public class ChatbotClient {
    private static final MediaType JSON_HEADER = MediaType.parse("application/json; charset=utf-8");
    public static final String SSE_URL = "https://alime-ws.aliyuncs.com/sse/paas4Json/%s/%s/%s/%s";
    private final ChatbotSSEToken token;

    static OkHttpClient httpClient = new OkHttpClient.Builder()
        .readTimeout(10, TimeUnit.SECONDS)
        .writeTimeout(10, TimeUnit.SECONDS)
        .connectTimeout(10, TimeUnit.SECONDS)
        .build();

    public ChatbotClient(String chatbotAgentKey, String aliyunAccessKey, String aliyunSecretKey) throws Exception {
        this.token = ChatbotSSETokenFetcher.getSseToken(aliyunAccessKey, aliyunSecretKey, chatbotAgentKey);
    }

    public void invokeLlm(ChatbotSSERequest reqData, ChannelMsgSender sender) throws Exception {

        long timeMillis = System.currentTimeMillis();
        String sign = ChatbotSignUtils.getSign(token.getStreamSecret(), String.valueOf(timeMillis));

        String url = String.format(SSE_URL, token.getAccessToken(), token.getChannelId(), sign, timeMillis);

        log.info("url:{}", url);

        EventSource.Factory factory = EventSources.createFactory(httpClient);

        String json = JSON.toJSONString(reqData);

        RequestBody body = RequestBody.create(JSON_HEADER, json);
        // 请求对象
        Request request = new Request.Builder().url(url).post(body).build();

        SSEEventSourceListener eventSourceListener = new SSEEventSourceListener(sender);

        factory.newEventSource(request, eventSourceListener);

    }
}
