package com.aliyun.chatbot.channel.client;

import javax.annotation.PostConstruct;

import com.aliyun.chatbot.channel.base.ChannelMsgSender;
import com.aliyun.chatbot.channel.chatbot.ChatbotClient;
import com.aliyun.chatbot.channel.chatbot.ChatbotSSERequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

/**
 * @author kelin
 * @date 2024/6/14
 */
@Component
@Slf4j
public class ChatProxyClient {
    private ChatbotClient chatbotClient;

    @Value("${CHATBOT_INSTANCE_ID}")
    private String CHATBOT_INSTANCE_ID;
    @Value("${CHATBOT_AGENT_KEY}")
    private String CHATBOT_AGENT_KEY;
    @Value("${ALIYUN_ACCESS_KEY}")
    private String ALIYUN_ACCESS_KEY;
    @Value("${ALIYUN_SECRET_KEY}")
    private String ALIYUN_SECRET_KEY;

    @PostConstruct
    public void init() throws Exception {
        chatbotClient = new ChatbotClient(CHATBOT_AGENT_KEY, ALIYUN_ACCESS_KEY, ALIYUN_SECRET_KEY);
    }

    public void chat(String sessionId, String text, SseEmitter sseEmitter) {
        log.info("receive message :{}", text);
        try {
            if (text != null) {
                //机器人接收消息内容
                JavaChatContext context = new JavaChatContext();
                context.setSessionId(sessionId);
                context.setQuery(text);
                context.setSseEmitter(sseEmitter);

                ChannelMsgSender sender = new JavaClientMsgSender(context);
                sender.preSengMsg();

                // 调用大模型
                ChatbotSSERequest reqData = ChatbotSSERequest.build(CHATBOT_INSTANCE_ID, text, sessionId);
                chatbotClient.invokeLlm(reqData, sender);
            }
        } catch (Exception e) {
            log.error("receive message by error:" + e.getMessage(), e);
        }
    }
}
